import 'package:flutter/material.dart';
import 'package:hgg_view/src/theme/h_colors.dart';
import 'package:wechat_assets_picker/wechat_assets_picker.dart';
import 'package:wechat_camera_picker/wechat_camera_picker.dart';

/// 图片选择工具
class XImagePickUtil {
  /// 拍照 only
  static Future<AssetEntity?> takePhoto(BuildContext context) async {
    final AssetEntity? asset = await CameraPicker.pickFromCamera(
      context,
      enableRecording: false, //是否可以录像
      resolutionPreset: ResolutionPreset.high,
      maximumRecordingDuration: const Duration(seconds: 10),
      theme: CameraPicker.themeData(HColors.app_accent_5A46BE),
    );
    return asset;
  }

  /// 选择单个图片
  static Future<AssetEntity?> single(BuildContext context) async {
    int gridCount = 4;
    final List<AssetEntity>? assets = await AssetPicker.pickAssets(
      context,
      maxAssets: 1,
      //最多选择的图片数量
      pageSize: 80 * gridCount,
      //分页加载时每页加载的资源数量。必须为网格数的倍数。 设置为null可以取消分页。
      pathThumbSize: 200,
      //选择器的缩略图大小
      gridCount: gridCount,
      //选择器网格数量
      requestType: RequestType.image,
      //选择器选择资源的类型
      themeColor: HColors.app_accent_5A46BE,
    );

    if (assets?.length == 1) {
      AssetEntity asset = assets![0];
      return asset;
    }
    return null;
  }

  /// 选择图片/视频,最多 9 个
  static Future<List<AssetEntity>?> multiple(
    BuildContext context, {
    int choose = 0,
    int maxLength = 9,
    RequestType requestType = RequestType.image,
  }) async {
    int gridCount = 4;
    final List<AssetEntity>? assets = await AssetPicker.pickAssets(
      context,
      maxAssets: maxLength - choose,
      //最多选择的图片数量
      pageSize: 80 * gridCount,
      //分页加载时每页加载的资源数量。必须为网格数的倍数。 设置为null可以取消分页。
      pathThumbSize: 200,
      //选择器的缩略图大小
      gridCount: gridCount,
      //选择器网格数量
      requestType: requestType,
      //选择器选择资源的类型
      themeColor: HColors.app_accent_5A46BE,
    );

    return assets;
  }

  /// 拍照 or 拍视频
  static Future<AssetEntity?> takePhotoOrVideo(BuildContext context) async {
    final AssetEntity? asset = await CameraPicker.pickFromCamera(
      context,
      enableRecording: true, //是否可以录像
      resolutionPreset: ResolutionPreset.high,
      maximumRecordingDuration: const Duration(seconds: 10),
      theme: CameraPicker.themeData(HColors.app_accent_5A46BE),
    );
    return asset;
  }
}
