import 'package:flutter/material.dart';
import 'package:hgg_view/hgg_view.dart';
import 'package:hgg_view/src/theme/h_colors.dart';
import 'package:wechat_assets_picker/wechat_assets_picker.dart';

import 'x_image_pick_util.dart';

/// 选择图片对话框:  拍照/相册
class XImagePickerDialog extends StatelessWidget {
  static show({
    required void Function(List<AssetEntity> assets) useAsset,
    int choose = 0,
    int maxLength = 1,
  }) {
    Get.bottomSheet(
      XImagePickerDialog._(useAsset, choose, maxLength),
      barrierColor: Colors.black26,
    );
  }

  const XImagePickerDialog._(this.useAsset, this.choose, this.maxLength);

  final Function(List<AssetEntity>) useAsset; //拍摄or选取图片,后执行
  final int choose; //已选择图片数
  final int maxLength; //可选择图片总数

  @override
  Widget build(BuildContext context) {
    return XContainer(
      height: 361,
      child: Column(
        children: <Widget>[
          _buildCameraOption(context),
          const XContainer(height: 1, color: HColors.FF_919191),
          _buildPhotoOption(context),
          _buildCancelOption()
        ],
      ),
    );
  }

  /// 相机
  Widget _buildCameraOption(BuildContext context) {
    return XInkWell(
      onTap: () async {
        final asset = await XImagePickUtil.takePhoto(context);
        if (asset != null) {
          useAsset([asset]);
        }
        Get.back();
      },
      borderRadius: BorderRadius.vertical(top: Radius.circular(D.w(40))),
      child: XContainer(
        height: 120,
        width: double.infinity,
        alignment: Alignment.center,
        color: HColors.FF_1F1B34,
        borderRadius: BorderRadius.vertical(top: Radius.circular(D.w(40))),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Icon(
              Icons.camera_alt,
              size: D.w(38),
              color: HColors.FF_F2F2F2,
            ),
            XText(
              '拍摄',
              color: HColors.FF_F2F2F2,
              fontSize: 34,
              fontWeight: XFontWeight.medium,
              padding: P.only(left: 39),
            ),
          ],
        ),
      ),
    );
  }

  /// 相册
  Widget _buildPhotoOption(BuildContext context) {
    return XInkWell(
      onTap: () async {
        final assets = await XImagePickUtil.multiple(context, choose: choose, maxLength: maxLength);
        if (assets != null && assets.isNotEmpty) {
          useAsset(assets);
        }
        Get.back();
      },
      child: XContainer(
        height: 120,
        width: double.infinity,
        alignment: Alignment.center,
        color: HColors.FF_1F1B34,
        child: Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Icon(
              Icons.photo,
              size: D.w(38),
              color: HColors.FF_F2F2F2,
            ),
            XText(
              '相册',
              color: HColors.FF_F2F2F2,
              fontSize: 34,
              fontWeight: XFontWeight.medium,
              padding: P.only(left: 39),
            ),
          ],
        ),
      ),
    );
  }

  /// 取消按钮
  Widget _buildCancelOption() {
    return XInkWell(
      onTap: Get.back,
      child: const XContainer(
        height: 120,
        width: double.infinity,
        alignment: Alignment.center,
        color: Color(0xFF765CFE),
        child: XText(
          '取消',
          color: HColors.FF_F2F2F2,
          fontSize: 34,
          fontWeight: XFontWeight.medium,
        ),
      ),
    );
  }
}
