import 'package:extended_image/extended_image.dart';
import 'package:flutter/material.dart';
import 'package:hgg_view/hgg_view.dart';
import 'package:hgg_view/src/theme/h_config.dart';

/// 大图浏览
class BigImagePage extends StatefulWidget {
  static go(
    List<String> pics, {
    int initialIndex = 0,
    String kIndex = '', //用于区别不同 tag
    bool isLocalImage = false,
  }) {
    Get.to(
        () => BigImagePage._(
              pics: pics,
              initialIndex: initialIndex,
              kIndex: kIndex,
              isLocalImage: isLocalImage,
            ),
        fullscreenDialog: true,
        opaque: false);
  }

  final int initialIndex; //初始索引
  final List<String> pics; //图片集合
  final String kIndex; // 标志,用于区分,防止 hero 动画 混乱(地址相同的时候)
  final bool isLocalImage;

  const BigImagePage._({
    required this.initialIndex,
    required this.pics,
    required this.kIndex,
    required this.isLocalImage,
  });

  @override
  BigImagePageState createState() => BigImagePageState();
}

class BigImagePageState extends State<BigImagePage> {
  late int currentIndex;
  var _showIndicator = false;

  @override
  void initState() {
    super.initState();
    if (widget.pics.length > 1) {
      _showIndicator = true;
    }
    currentIndex = widget.initialIndex;
  }

  @override
  Widget build(BuildContext context) {
    return ExtendedImageSlidePage(
      slideAxis: SlideAxis.both,
      slideType: SlideType.onlyImage,
      onSlidingPage: (state) {
        if (widget.pics.length > 1) {
          var showIndicator = !state.isSliding;
          if (showIndicator != _showIndicator) {
            // do not setState directly here, the image state will change,
            // you should only notify the widgets which are needed to change
            // setState(() {
            // _showSwiper = showSwiper;
            // });
            setState(() {
              _showIndicator = showIndicator;
            });
//          rebuildSwiper.add(_showSwiper);
          }
        }
      },
      child: Material(
        color: Colors.transparent,
        shadowColor: Colors.transparent,
        child: Stack(
          children: <Widget>[
            ExtendedImageGesturePageView.builder(
              physics: const BouncingScrollPhysics(),
              itemBuilder: (BuildContext context, int index) {
                var image = widget.pics[index];
                XLog.d(image);
                XLog.d(widget.isLocalImage);

                if (widget.isLocalImage) {
                  return GestureDetector(
                    onTap: Get.back,
                    child: ExtendedImage.file(
                      File(image),
                      fit: BoxFit.contain,
                      enableSlideOutPage: true,
                      mode: ExtendedImageMode.gesture,
                      loadStateChanged: _loadStateChanged,
                      heroBuilderForSlidingPage: (Widget result) =>
                          _heroConfig(index, result),
                      initGestureConfigHandler: _initGestureConfig,
                    ),
                  );
                }
                if (!image.startsWith('http')) {
                  image = HConfig.imagePrefix + image.trim();
                }

                return GestureDetector(
                  onTap: Get.back,
                  child: ExtendedImage.network(
                    image,
                    fit: BoxFit.contain,
                    enableSlideOutPage: true,
                    mode: ExtendedImageMode.gesture,
                    loadStateChanged: _loadStateChanged,
                    heroBuilderForSlidingPage: (Widget result) =>
                        _heroConfig(index, result),
                    initGestureConfigHandler: _initGestureConfig,
                  ),
                );
              },
              itemCount: widget.pics.length,
              onPageChanged: (int index) {
                setState(() {
                  currentIndex = index;
                });
              },
              controller: ExtendedPageController(
                initialPage: currentIndex,
              ),
              scrollDirection: Axis.horizontal,
            ),
            Align(
              alignment: const Alignment(0.0, 0.9),
              child: Visibility(
                visible: _showIndicator,
                child: Indicator(
                  itemCount: widget.pics.length,
                  currentIndex: currentIndex,
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  Widget _heroConfig(int index, Widget result) {
    if (index < widget.pics.length) {
      return Hero(
        tag: widget.pics[index] + widget.kIndex,
        child: result,
      );
    } else {
      return result;
    }
  }

  GestureConfig _initGestureConfig(state) {
    return GestureConfig(
      minScale: 0.3,
      maxScale: 4,
      inPageView: true,
      initialScale: 1.0,
      cacheGesture: false,
    );
  }

  Widget? _loadStateChanged(state) {
    if (state.extendedImageLoadState == LoadState.failed) {
      return Container(
        alignment: Alignment.center,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Icon(
              Icons.wb_cloudy_outlined,
              size: D.w(120),
              color: Colors.grey[500],
            ),
            GestureDetector(
              onTap: () {
                state.reLoadImage();
              },
              child: Text(
                '获取失败,点击重试',
                style: TextStyle(
                    color: Colors.grey[400],
                    fontSize: D.sp(28),
                    fontWeight: FontWeight.w400),
              ),
            )
          ],
        ),
      );
    }
    return null;
  }
}

class Indicator extends StatelessWidget {
  Indicator({
    Key? key,
    this.itemCount = 0,
    this.currentIndex = 0,
  }) : super(key: key);

  /// 指示器的个数
  final int itemCount;

  /// 当前选中的页面
  final int currentIndex;

  /// 普通的颜色
  final Color normalColor = Colors.grey;

  /// 选中的颜色
  final Color? selectedColor = Colors.grey[200];

  /// 点的大小
  final double size = 8.0;

  /// 点的间距
  final double spacing = 4.0;

  /// 点的Widget
  Widget _buildIndicator(
      int index, int pageCount, double dotSize, double spacing) {
    // 是否是当前页面被选中
    bool isCurrentPageSelected = index == currentIndex;

    return SizedBox(
      height: size,
      width: size + (2 * spacing),
      child: Center(
        child: Material(
          color: isCurrentPageSelected ? selectedColor : normalColor,
          type: MaterialType.circle,
          child: SizedBox(
            width: dotSize,
            height: dotSize,
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: List<Widget>.generate(itemCount, (int index) {
        return _buildIndicator(index, itemCount, size, spacing);
      }),
    );
  }
}
