import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:hgg_view/hgg_view.dart';
import 'package:hgg_view/src/theme/h_colors.dart';

import '../x_container.dart';
import '../x_ink_well.dart';

/// 应用 Button
class XButton extends StatelessWidget {
  const XButton({
    Key? key,
    this.width,
    required this.height,
    this.radius,
    Color? enableColor,
    required this.text,
    this.fontColor,
    this.fontSize,
    this.fontWeight,
    required this.onPressed,
    this.disableColor,
    this.splashColor,
    this.enable = true,
    this.elevation = 2,
    this.beforeTextImgPath,
    this.afterTextImgPath,
    this.imgSize = 24,
    this.isLoading = false,
    this.padding,
  })
      : enableColor = enableColor ?? HColors.FF_765CFE,
        super(key: key);

  final String? text;
  final VoidCallback onPressed;
  final bool enable;
  final double height;
  final double? width;
  final double? radius;
  final double elevation; //高度
  final Color enableColor; //启动颜色
  final Color? disableColor; //禁用颜色
  final Color? splashColor; //波纹颜色
  final String? beforeTextImgPath; //前置图片
  final String? afterTextImgPath; //后缀图片
  final double imgSize;
  final bool isLoading; //是否加载中
  final Color? fontColor;
  final double? fontSize;
  final FontWeight? fontWeight;
  final EdgeInsetsGeometry? padding;

  @override
  Widget build(BuildContext context) {
    Widget widget = XText(
      text ?? '',
      color: fontColor ?? Colors.white,
      fontSize: fontSize ?? 24,
      fontWeight: fontWeight ?? XFontWeight.medium,
    );

    if (beforeTextImgPath != null) {
      widget = Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          XImageAsset(
            beforeTextImgPath!,
            height: imgSize,
            width: imgSize,
          ),
          G.h(18),
          widget
        ],
      );
    }

    if (afterTextImgPath != null) {
      widget = Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          widget,
          G.h(18),
          XImageAsset(
            afterTextImgPath!,
            height: imgSize,
            width: imgSize,
          ),
        ],
      );
    }

    if (isLoading) {
      widget = Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          const CupertinoTheme(
            data: CupertinoThemeData(brightness: Brightness.dark),
            child: CupertinoActivityIndicator(radius: 8),
          ),
          G.h(18),
          widget,
        ],
      );
    }

    widget = Material(
      elevation: elevation,
      borderRadius: radius != null ? XRadius.circular(radius) : XRadius
          .circular(height / 2),
      color: enable ? enableColor : disableColor ?? enableColor.withOpacity(.4),
      child: InkWell(
        onTap: (enable && !isLoading) ? onPressed : null,
        borderRadius: radius != null ? XRadius.circular(radius) : XRadius
            .circular(height / 2),
        splashColor: splashColor ?? enableColor.withRed(enableColor.red + 8),
        child: XContainer(
          width: width,
          height: height,
          alignment: Alignment.center,
          child: widget,
        ),
      ),
    );

    if (padding != null) {
      widget = Padding(
        padding: padding!,
        child: widget,
      );
    }

    return widget;
  }
}

class XIconButton extends StatelessWidget {
  final Widget child;
  final GestureTapCallback onTap;
  final double padding;
  final Color? splashColor;

  const XIconButton({
    required this.child,
    required this.onTap,
    this.padding = 24,
    this.splashColor,
    Key? key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return XInkWell(
      splashColor: splashColor ?? Colors.deepPurple.withOpacity(0.1),
      borderRadius: XRadius.circular(padding + 30),
      onTap: onTap,
      padding: padding,
      child: child,
    );
  }
}
