import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:hgg_view/hgg_view.dart';
import 'package:hgg_view/src/theme/h_colors.dart';

/// 应用 Button
class XOutlineButton extends StatelessWidget {
  const XOutlineButton({
    Key? key,
    this.width,
    required this.height,
    this.radius,
    this.borderWidth,
    Color? enableColor,
    required this.text,
    this.fontColor,
    this.fontSize,
    this.fontWeight,
    required this.onPressed,
    this.disableColor,
    this.splashColor,
    this.enable = true,
    this.elevation = 2,
    this.beforeTextImgPath,
    this.afterTextImgPath,
    this.isLoading = false,
    this.padding,
  })  : enableColor = enableColor ?? HColors.FF_765CFE,
        super(key: key);

  final String text;
  final VoidCallback onPressed;
  final bool enable;
  final double height;
  final double? width;
  final double? radius;
  final double? borderWidth; //边框宽度
  final double elevation; //高度
  final Color enableColor; //启动颜色
  final Color? disableColor; //禁用颜色
  final Color? splashColor; //波纹颜色
  final String? beforeTextImgPath; //前置图片
  final String? afterTextImgPath; //后缀图片
  final bool isLoading; //是否加载中
  final Color? fontColor;
  final double? fontSize;
  final FontWeight? fontWeight;
  final EdgeInsetsGeometry? padding;

  @override
  Widget build(BuildContext context) {
    Widget widget = XText(
      text,
      color: fontColor ?? HColors.FF_765CFE,
      fontSize: fontSize ?? 24,
      fontWeight: fontWeight ?? XFontWeight.medium,
    );

    if (beforeTextImgPath != null) {
      widget = Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          XImageAsset(
            beforeTextImgPath!,
            height: 24,
          ),
          G.h(18),
          widget
        ],
      );
    }

    if (afterTextImgPath != null) {
      widget = Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          widget,
          G.h(18),
          XImageAsset(
            afterTextImgPath!,
            height: 24,
          ),
        ],
      );
    }

    if (isLoading) {
      widget = Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          const CupertinoTheme(
            data: CupertinoThemeData(brightness: Brightness.dark),
            child: CupertinoActivityIndicator(radius: 8),
          ),
          G.h(18),
          widget,
        ],
      );
    }

    widget = XInkWell(
      onTap: (enable && !isLoading) ? onPressed : null,
      borderRadius: radius != null ? XRadius.circular(radius) : XRadius.circular(height / 2),
      splashColor: splashColor ?? enableColor.withOpacity(.1),
      child: XContainer(
        width: width,
        height: height,
        alignment: Alignment.center,
        radius: radius ?? (height / 2),
        border: Border.all(
          color: enableColor,
          width: D.w(borderWidth ?? 1),
        ),
        child: widget,
      ),
    );

    if (padding != null) {
      widget = Padding(
        padding: padding!,
        child: widget,
      );
    }

    return widget;
  }
}
