import 'dart:math';

import 'package:flutter/material.dart';
import 'package:hgg_view/hgg_view.dart';
import 'package:hgg_view/src/generated/r.dart';
import 'package:hgg_view/src/theme/h_colors.dart';

/// 选择按钮
class XSelectButton extends StatelessWidget {
  final bool isOpenSelect; //是否打开选择框,用于三角动画
  final String selectShow; //选择的内容展示
  final String hint; //选择前的文字提示
  final GestureTapCallback onTap; //点击控件
  final double? width; //选择框宽度
  final Color? backgroundColor; //选择框背景色
  final double? fontSize; //选择展示文字大小
  final Color? fontColor; //选择展示文字颜色

  const XSelectButton(
      {Key? key,
      required this.isOpenSelect,
      required this.selectShow,
      required this.onTap,
      this.hint = '请选择',
      this.width,
      this.backgroundColor,
      this.fontSize,
      this.fontColor})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return XInkWell(
      onTap: onTap,
      child: XContainer(
        width: width,
        constraints: BoxConstraints(minHeight: D.h(64)),
        color: backgroundColor,
        border: Border.all(color: HColors.FF_423F4F, width: D.w(1)),
        child: Row(
          children: <Widget>[
            Expanded(
              child: XText(
                selectShow.isEmpty ? hint : selectShow,
                color: selectShow.isEmpty
                    ? (fontColor ?? HColors.FF_B0B2D8).withOpacity(.5)
                    : (fontColor ?? HColors.FF_B0B2D8),
                fontSize: fontSize ?? 28,
                fontWeight: XFontWeight.regular,
                padding: P.only(left: 20),
              ),
            ),
            const XContainer(
              color: HColors.FF_423F4F,
              height: 64,
              width: 1,
            ),
            XContainer(
              width: 24,
              height: 15,
              margin: P.hv(horizontal: 20),
              child: _AnimationArrow(isSelected: isOpenSelect),
            ),
          ],
        ),
      ),
    );
  }
}

///三角旋转动画
class _AnimationArrow extends StatelessWidget {
  final bool isSelected;

  const _AnimationArrow({this.isSelected = false});

  @override
  Widget build(BuildContext context) {
    return TweenAnimationBuilder<double>(
      tween: Tween(begin: 1, end: isSelected ? 1 : 0),
      duration: const Duration(milliseconds: 200),
      builder: (context, value, child) {
        return Transform.rotate(
          //pi为180° 此处为三角顺时针旋转180° 整个圆 如果不需要整个圆旋转 去掉isSelected的参数
          angle: pi * value,
          child: Image.asset(
            R.icon_arrow_down,
            gaplessPlayback: true,
            package: HConfig.package,
          ),
        );
      },
    );
  }
}
