import 'package:flutter/material.dart';
import 'package:hgg_view/hgg_view.dart';
import 'package:hgg_view/src/theme/h_colors.dart';

/// 浮动按钮
class XFloat extends StatelessWidget {
  const XFloat({
    Key? key,
    required this.child,
    required this.options,
    required this.currentOption,
    required this.onChange,
  }) : super(key: key);

  final Widget child; //被点击的控件
  final List<String> options; //选项
  final String currentOption; //当前选项
  final void Function(String) onChange; //选择改变

  @override
  Widget build(BuildContext context) {
    final GlobalKey anchorKey = GlobalKey();
    return WillPopScope(
      onWillPop: () async {
        XFloatContent.dismiss();
        return true;
      },
      child: InkWell(
        onTap: () {
          XFloatContent.show(
            context,
            tapWidgetKey: anchorKey,
            child: Column(
              children: options.map((option) {
                return XInkWell(
                  onTap: () {
                    XFloatContent.dismiss();
                    onChange(option);
                  },
                  child: XContainer(
                    width: 158,
                    height: 80,
                    color: option == currentOption ? HColors.FF_765CFE : HColors.FF_2F2D3A,
                    alignment: Alignment.center,
                    child: XText(
                      option,
                      color: option == currentOption ? HColors.FF_E6E6E6 : HColors.FF_B0B2D8,
                      fontSize: 24,
                      fontWeight: XFontWeight.medium,
                    ),
                  ),
                );
              }).toList(),
            ),
          );
        },
        child: Container(key: anchorKey, child: child),
      ),
    );
  }
}

/// 通用悬浮控件
class XFloatContent extends StatelessWidget {
  static OverlayEntry? _entry;

  static show(
    BuildContext context, {
    required Widget child,
    GlobalKey? tapWidgetKey,
    double? dx,
  }) {
    Offset? offset;
    if (tapWidgetKey != null) {
      final renderObject = tapWidgetKey.currentContext?.findRenderObject();
      if (renderObject != null && renderObject is RenderBox) {
        //获得控件正下方的坐标
        offset = renderObject.localToGlobal(Offset(dx ?? 0.0, renderObject.size.height));
      }
    }
    if (offset != null) {
      _entry = OverlayEntry(
          builder: (BuildContext context) => XFloatContent(
                child: child,
                offset: offset,
              ));
    }
    if (_entry != null) Overlay.of(context)?.insert(_entry!);
  }

  static dismiss() {
    _entry?.remove();
  }

  final Widget child;
  final Offset? offset;

  const XFloatContent({Key? key, required this.child, this.offset}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Stack(
      clipBehavior: Clip.none,
      children: [
        GestureDetector(
          onTap: dismiss,
          child: Container(color: Colors.transparent),
        ),
        _buildFloatContent()
      ],
    );
  }

  ///浮动控件
  Widget _buildFloatContent() {
    return Positioned(
      left: offset?.dx,
      top: offset?.dy,
      child: Material(color: Colors.transparent, child: child),
    );
  }
}
