import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:hgg_view/hgg_view.dart';
import 'package:hgg_view/src/generated/r.dart';

/// 显示评分控件
class XScoreWidget extends StatefulWidget {
  final int count; // star 数
  final double maxRating; // 最高分
  final double? value; // 当前分值
  final double size; // Star size
  final double padding; // Space between the stars
  final String? nomalImage; // empty star image assets url
  final String? selectImage; // full star image assets url
  final bool selectAble; // Whether the score can be modified by sliding
  final ValueChanged<String>? onRatingUpdate; // Callbacks when ratings change

  const XScoreWidget({
    Key? key,
    double value = 5,
    this.maxRating = 5.0,
    this.count = 5,
    this.size = 22,
    this.nomalImage,
    this.selectImage,
    this.padding = 6,
    this.selectAble = false,
    this.onRatingUpdate,
  })  : value = (value > maxRating) ? maxRating : value,
        super(key: key);

  @override
  _XScoreWidgetState createState() => _XScoreWidgetState();
}

class _XScoreWidgetState extends State<XScoreWidget> {
  double? value;
  double? padding;
  double? size;

  @override
  Widget build(BuildContext context) {
    padding = D.w(widget.padding);
    size = D.w(widget.size);

    return Listener(
      child: buildRowRating(),
      onPointerDown: (PointerDownEvent event) {
        double x = event.localPosition.dx;
        if (x < 0) x = 0;
        pointValue(x);
      },
      onPointerMove: (PointerMoveEvent event) {
        double x = event.localPosition.dx;
        if (x < 0) x = 0;
        pointValue(x);
      },
      onPointerUp: (_) {},
      behavior: HitTestBehavior.deferToChild,
    );
  }

  pointValue(double dx) {
    if (!widget.selectAble) {
      return;
    }

    if (dx >= size! * widget.count + padding! * (widget.count - 1)) {
      value = widget.maxRating;
    } else {
      for (double i = 1; i < widget.count + 1; i++) {
        if (dx > size! * i + padding! * (i - 1) && dx < size! * i + padding! * i) {
          value = i * (widget.maxRating / widget.count);
          break;
        } else if (dx > size! * (i - 1) + padding! * (i - 1) && dx < size! * i + padding! * i) {
          value = (dx - padding! * (i - 1)) / (size! * widget.count) * widget.maxRating;
          break;
        }
      }
    }
    setState(() {
      widget.onRatingUpdate?.call(value!.toStringAsFixed(1));
    });
  }

  int fullStars() {
    if (value != null) {
      return (value! / (widget.maxRating / widget.count)).floor();
    }
    return 0;
  }

  double star() {
    if (value != null) {
      if (widget.count / fullStars() == widget.maxRating / value!) {
        return 0;
      }
      return (value! % (widget.maxRating / widget.count)) / (widget.maxRating / widget.count);
    }
    return 0;
  }

  /// 评分数
  List<Widget> buildRow() {
    int full = fullStars();
    List<Widget> children = [];

    // 评分图片,前景
    XImageAsset image;

    image = XImageAsset(
      widget.selectImage ?? R.icon_star,
      width: widget.size,
      height: widget.size,
      package: widget.selectImage == null ? HConfig.package : null,
    );

    for (int i = 0; i < full; i++) {
      children.add(image);
      if (i < widget.count - 1) {
        children.add(
          SizedBox(
            width: padding,
          ),
        );
      }
    }
    if (full < widget.count) {
      children.add(ClipRect(
        clipper: SMClipper(rating: star() * size!),
        child: image,
      ));
    }

    return children;
  }

  /// 评分背景
  List<Widget> buildNomalRow() {
    List<Widget> children = [];
    for (int i = 0; i < widget.count; i++) {
      // 评分图片,背景
      XImageAsset image;

      image = XImageAsset(
        widget.nomalImage ?? R.icon_star_back,
        width: widget.size,
        height: widget.size,
        package: widget.selectImage == null ? HConfig.package : null,
      );

      children.add(image);
      if (i < widget.count - 1) {
        children.add(SizedBox(
          width: padding,
        ));
      }
    }
    return children;
  }

  Widget buildRowRating() {
    return Stack(
      children: <Widget>[
        Row(
          children: buildNomalRow(),
        ),
        Row(
          children: buildRow(),
        )
      ],
    );
  }

  @override
  void initState() {
    super.initState();
    value = widget.value;
  }
}

class SMClipper extends CustomClipper<Rect> {
  final double rating;

  SMClipper({required this.rating});

  @override
  Rect getClip(Size size) {
    return Rect.fromLTRB(0.0, 0.0, rating, size.height);
  }

  @override
  bool shouldReclip(SMClipper oldClipper) {
    return rating != oldClipper.rating;
  }
}
