import 'package:extended_text_field/extended_text_field.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:hgg_view/hgg_view.dart';
import 'package:hgg_view/src/theme/h_colors.dart';

/// 输入框
class XTextField extends StatelessWidget {
  final bool? enabled;
  final TextInputAction? textInputAction;
  final bool isDone;
  final TextInputType textInputType;
  final int? maxLength;
  final int maxLines;
  final int minLines;
  final bool autofocus;
  final TextStyle? textStyle;
  final String? hintText;
  final TextStyle? hintStyle;
  final EdgeInsetsGeometry? contentPadding;
  final ValueChanged<String>? onChanged;
  final ValueChanged<String>? onSubmitted;
  final TextEditingController? controller;
  final FocusNode? focusNode;
  final TextAlign? textAlign;

  final double? height;
  final double? minHeight;
  final double? width;
  final Color? color;
  final double? radius;
  final Color? borderColor;

  final SpecialTextSpanBuilder? specialTextSpanBuilder;

  const XTextField({
    this.enabled,
    this.textInputAction,
    this.isDone = false,
    this.textInputType = TextInputType.text,
    this.maxLength,
    this.minLines = 1,
    this.maxLines = 1,
    this.autofocus = false,
    this.textStyle,
    this.hintText,
    this.hintStyle,
    this.contentPadding,
    this.onChanged,
    this.onSubmitted,
    this.controller,
    this.focusNode,
    this.height,
    this.minHeight,
    this.width,
    this.color,
    this.radius,
    this.borderColor,
    this.specialTextSpanBuilder,
    this.textAlign,
    Key? key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    List<TextInputFormatter>? inputFormatters;
    if (textInputType == TextInputType.phone) {
      // 手机号限制格式为0到9(白名单)
      inputFormatters = [FilteringTextInputFormatter.allow(RegExp('[0-9]'))];
    }
    if (textInputType == TextInputType.number) {
      // 数字限制格式为0到9(白名单)，包括小数点
      inputFormatters = [FilteringTextInputFormatter.allow(RegExp('[0-9,.]'))];
    }

    Widget widget;
    if (specialTextSpanBuilder == null) {
      widget = TextField(
        enabled: enabled,
        controller: controller,
        focusNode: focusNode,
        onChanged: onChanged,
        onSubmitted: onSubmitted,
        scrollPhysics: const BouncingScrollPhysics(),
        keyboardType: textInputType,
        textInputAction: textInputAction ?? (isDone ? TextInputAction.done : TextInputAction.next),
        cursorColor: HColors.app_accent_5A46BE,
        maxLength: textInputType == TextInputType.phone ? 11 : maxLength,
        maxLines: maxLines,
        minLines: minLines,
        autofocus: autofocus,
        style: textStyle,
        textAlign: textAlign ?? TextAlign.start,
        inputFormatters: inputFormatters,
        decoration: InputDecoration(
          contentPadding: contentPadding,
          hintText: hintText,
          hintStyle: hintStyle,
          counterText: '',
          border: InputBorder.none,
          isDense: true,
        ),
      );
    } else {
      widget = ExtendedTextField(
        enabled: enabled,
        specialTextSpanBuilder: specialTextSpanBuilder,
        controller: controller,
        focusNode: focusNode,
        onChanged: onChanged,
        onSubmitted: onSubmitted,
        scrollPhysics: const BouncingScrollPhysics(),
        keyboardType: textInputType,
        textInputAction: textInputAction ?? (isDone ? TextInputAction.done : TextInputAction.next),
        cursorColor: HColors.app_accent_5A46BE,
        maxLength: textInputType == TextInputType.phone ? 11 : maxLength,
        maxLines: maxLines,
        minLines: minLines,
        autofocus: autofocus,
        style: textStyle,
        textAlign: textAlign ?? TextAlign.start,
        inputFormatters: inputFormatters,
        decoration: InputDecoration(
          contentPadding: contentPadding,
          hintText: hintText,
          hintStyle: hintStyle,
          counterText: '',
          border: InputBorder.none,
          isDense: true,
        ),
      );
    }

    if (height != null || minHeight != null) {
      double defaultRadius = 0;
      if (minHeight != null) {
        defaultRadius = minHeight! / 2;
      } else if (height != null) {
        defaultRadius = height! / 2;
      }

      widget = Container(
          height: D.h(height),
          width: D.w(width),
          constraints: minHeight != null ? BoxConstraints(minHeight: D.h(minHeight)) : null,
          alignment: Alignment.center,
          decoration: BoxDecoration(
              color: color,
              borderRadius: XRadius.circular(radius ?? defaultRadius),
              border: borderColor != null ? Border.all(color: borderColor!, width: D.h(1)) : null),
          child: widget);
    }

    return widget;
  }
}
