import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:hgg_view/hgg_view.dart';
import 'package:hgg_view/src/theme/h_colors.dart';

// 应用 AppBar
class XAppBar extends StatelessWidget implements PreferredSizeWidget {
  final String? titleText;
  final bool isBack;
  final PreferredSizeWidget? bottom;
  final bool showBottomLine;
  final List<Widget>? actions;
  final String? leftText;
  final Color? backgroundColor;
  final VoidCallback? onTapLeft;
  final String? actionText;
  final VoidCallback? onTapTextAction;
  final SystemUiOverlayStyle? systemOverlayStyle;

  const XAppBar(
    this.titleText, {
    Key? key,
    this.bottom,
    this.showBottomLine = true,
    this.isBack = true,
    this.actions,
    this.leftText,
    this.backgroundColor,
    this.onTapLeft,
    this.actionText,
    this.onTapTextAction,
    this.systemOverlayStyle,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return AppBar(
      backgroundColor: backgroundColor,
      systemOverlayStyle: systemOverlayStyle ?? SystemUiOverlayStyle.light,
      //默认白色文字
      title: XText(
        titleText ?? '',
        fontSize: 36,
        color: HColors.FF_ECECEC,
        fontWeight: XFontWeight.medium,
      ),
      centerTitle: true,
      elevation: 0,
      actions: actions ??
          (actionText != null
              ? [
                  Row(
                    children: [
                      TextButton(
                        onPressed: onTapTextAction,
                        child: XText(
                          actionText,
                          color: HColors.FF_765CFE,
                          fontSize: 32,
                          fontWeight: XFontWeight.medium,
                        ),
                      ),
                    ],
                  )
                ]
              : null),
      leadingWidth: leftText != null ? D.w(100) : 46,
      leading: leftText != null
          ? Center(
              child: TextButton(
                onPressed: onTapLeft,
                child: XText(
                  leftText ?? '',
                  color: HColors.FF_CCCCCC,
                  fontSize: 32,
                  fontWeight: XFontWeight.medium,
                ),
              ),
            )
          : Visibility(
              visible: isBack,
              child: XIconButton(
                child: Icon(
                  Icons.arrow_back_ios,
                  color: HColors.FF_6C50FE,
                  size: D.w(36),
                ),
                onTap: Get.back,
              ),
            ),
      // backgroundColor: XColors.app_bar,
      bottom: bottom ??
          (showBottomLine
              ? PreferredSize(
                  child: G.line,
                  preferredSize: Size.fromHeight(D.w(1)),
                )
              : null),
      primary: true,
    );
  }

  // appbar 高度 kToolbarHeight
  @override
  Size get preferredSize => Size.fromHeight(46 + (bottom?.preferredSize.height ?? 0.0));
}

// 应用 自定义 AppBar
class XCustomAppBar extends StatelessWidget implements PreferredSizeWidget {
  final double _statusBarHeight;
  final double height;
  final Widget child;
  final Color? color;

  XCustomAppBar(
    BuildContext _context, {
    Key? key,
    required this.height,
    required this.child,
    this.color,
  })  : _statusBarHeight = MediaQuery.of(_context).padding.top,
        super(key: key);

  @override
  Widget build(BuildContext context) {
    Widget widget = Container(
      height: D.w(height),
      margin: EdgeInsets.only(top: _statusBarHeight),
      child: child,
    );

    if (color != null) {
      widget = ColoredBox(color: color!, child: widget);
    }

    return widget;
  }

  @override
  Size get preferredSize => Size.fromHeight(_statusBarHeight + D.w(height));
}
