import 'package:flutter/material.dart';
import 'package:flutter_swiper_null_safety/flutter_swiper_null_safety.dart';
import 'package:hgg_view/hgg_view.dart';
import 'package:hgg_view/src/pages/big_image_page.dart';
import 'package:hgg_view/src/theme/h_colors.dart';

import 'x_image_network.dart';

/// 轮播图
class XBanner extends StatelessWidget {
  final List<String> imgs;
  final double height; //图片高度
  final double dotSpace; //指示点间距
  final double marginBottom; //指示点距离下方边界高度
  final BoxFit fit;
  final double? radius; //圆角
  final SwiperOnTap? onTap;

  const XBanner({
    Key? key,
    required this.imgs,
    required this.height,
    this.dotSpace = 23,
    this.marginBottom = 21,
    this.fit = BoxFit.cover,
    this.radius,
    this.onTap,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (imgs.isEmpty) {
      return G.empty;
    }

    Widget view = Swiper(
      onTap:
          onTap ?? (index) => BigImagePage.go(imgs, initialIndex: index, kIndex: index.toString()),
      // viewportFraction: 0.9,
      scale: 1,
      loop: imgs.length > 1,
      layout: SwiperLayout.DEFAULT,
      itemCount: imgs.length,
      autoplay: false,
      pagination: _buildSwiperPagination(),
      indicatorLayout: PageIndicatorLayout.COLOR,
      itemBuilder: (context, index) {
        return Hero(
          tag: imgs[index] + index.toString(),
          child: XImageNetwork(
            imgs[index],
            fit: fit,
          ),
        );
      },
    );

    if (radius != null) {
      view = ClipRRect(
        borderRadius: XRadius.circular(radius),
        child: view,
      );
    }
    return XContainer(
      height: height,
      radius: 40,
      child: view,
    );
  }

  /// 指示点UI配置
  SwiperPagination _buildSwiperPagination() {
    return SwiperPagination(
      margin: P.only(bottom: marginBottom),
      alignment: Alignment.bottomCenter,
      builder: DotSwiperPaginationBuilder(
        size: D.w(14),
        space: D.w(dotSpace),
        color: HColors.FF_FEFEFE.withOpacity(.4),
        activeColor: HColors.FF_FEFEFE,
      ),
    );
  }
}
