import 'package:flutter/material.dart';

/// 应用 虚线
class XDashLine extends StatelessWidget {
  final double dashHeight;
  final Color color;
  final double dashWidth;
  final Axis direction; //方向
  final double? length; //线长

  const XDashLine({
    Key? key,
    this.color = Colors.black,
    this.dashHeight = 2,
    this.dashWidth = 2,
    this.direction = Axis.horizontal,
    this.length,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (context, BoxConstraints constraints) {
      int dashCount; //点数
      double lineLength; //线长
      if (direction == Axis.horizontal) {
        lineLength = length ?? constraints.constrainWidth();
        dashCount = (lineLength / (1.5 * dashWidth)).floor();
      } else {
        lineLength = length ?? constraints.constrainHeight();
        dashCount = (lineLength / (1.5 * dashHeight)).floor();
      }

      return Row(
        children: <Widget>[
          SizedBox(
            width: direction == Axis.horizontal ? lineLength : dashWidth,
            height: direction == Axis.horizontal ? dashHeight : lineLength,
            child: Flex(
              children: List.generate(
                dashCount,
                (_) {
                  return SizedBox(
                    width: dashWidth,
                    height: dashHeight,
                    child: DecoratedBox(
                      decoration: BoxDecoration(color: color),
                    ),
                  );
                },
              ),
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              direction: direction,
            ),
          ),
        ],
      );
    });
  }
}
