import 'package:extended_text_field/extended_text_field.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:hgg_view/hgg_view.dart';
import 'package:hgg_view/src/generated/r.dart';

///点击之后
typedef CallClick = void Function(Expression expression);

/// 表情组件
class XExpression extends StatelessWidget {
  final int crossAxisCount; // 一行表情数量
  final double mainAxisSpacing; // 纵轴间距
  final double crossAxisSpacing; // 横轴间距
  final double childAspectRatio; // 子Widget宽高比例
  final double padding; // 单个表情 padding

  final CallClick _callClick;

  const XExpression(this._callClick,
      {Key? key,
      this.crossAxisCount = 8,
      this.mainAxisSpacing = 2,
      this.crossAxisSpacing = 2,
      this.childAspectRatio = 1,
      this.padding = 10})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return GridView.custom(
      padding: P.all(20),
      physics: const BouncingScrollPhysics(),
      gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
        crossAxisCount: crossAxisCount,
        mainAxisSpacing: mainAxisSpacing,
        crossAxisSpacing: crossAxisSpacing,
      ),
      childrenDelegate: SliverChildBuilderDelegate(
        (context, position) {
          return _getExpressionItemContainer(position);
        },
        childCount: ExpressionData.EXPRESSION_SIZE,
      ),
    );
  }

  ///获取表情列表
  _getExpressionItemContainer(int index) {
    final expressionPath = ExpressionData.expressionPath[index];
    return XSingleExpression(expressionPath, padding, _callClick);
  }
}

///单个表情
class XSingleExpression extends StatelessWidget {
  final Expression expression;

  ///大小比例,值的大小与表情的大小成反比
  final double padding;

  final CallClick _callClick;

  const XSingleExpression(this.expression, this.padding, this._callClick, {Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return TextButton(
      style: TextButton.styleFrom(
        padding: P.all(padding),
      ),
      onPressed: () {
        _callClick(expression);
      },
      child: XImageAsset(
        expression.path,
        package: HConfig.package,
      ),
    );
  }
}

///表情对象
class Expression {
  final String name;
  final String path;
  final bool isEmoji; // 标识是否是emoji表情,true是,默认false

  Expression(this.name, this.path, {this.isEmoji = false});
}

///数据类
class ExpressionData {
  ///表情路径
  static final List<Expression> expressionPath = [
    Expression('微笑', R.expression_hehe),
    Expression('撇嘴', R.expression_piezui),
    Expression('色', R.expression_se),
    Expression('发呆', R.expression_fadai),
    Expression('得意', R.expression_deyi),
    Expression('流泪', R.expression_liulei),
    Expression('害羞', R.expression_haixiu),
    Expression('闭嘴', R.expression_bizui),
    Expression('睡', R.expression_shui),
    Expression('大哭', R.expression_daku),
    Expression('尴尬', R.expression_ganga),
    Expression('发怒', R.expression_fanu),
    Expression('调皮', R.expression_tiaopi),
    Expression('呲牙', R.expression_ciya),
    Expression('惊讶', R.expression_jingya),
    Expression('难过', R.expression_nanguo),
    Expression('囧', R.expression_jiong),
    Expression('抓狂', R.expression_zhuakuang),
    Expression('吐', R.expression_tu),
    Expression('偷笑', R.expression_touxiao),
    Expression('愉快', R.expression_yukuai),
    Expression('白眼', R.expression_baiyan),
    Expression('傲慢', R.expression_aoman),
    Expression('困', R.expression_kun),
    Expression('惊恐', R.expression_jingkong),
    Expression('流汗', R.expression_liuhan),
    Expression('憨笑', R.expression_hanxiao),
    Expression('悠闲', R.expression_youxian),
    Expression('奋斗', R.expression_fendou),
    Expression('咒骂', R.expression_zhouma),
    Expression('疑问', R.expression_yiwen),
    Expression('嘘', R.expression_xu),
    Expression('晕', R.expression_yun),
    Expression('衰', R.expression_sui),
    Expression('骷髅', R.expression_kulou),
    Expression('敲打', R.expression_qiaoda),
    Expression('再见', R.expression_zaininmadejian),
    Expression('擦汗', R.expression_cahan),
    Expression('抠鼻', R.expression_koubi),
    Expression('鼓掌', R.expression_guzhang),
    Expression('坏笑', R.expression_huaixiao),
    Expression('左哼哼', R.expression_zuohengheng),
    Expression('右哼哼', R.expression_youhengheng),
    Expression('哈欠', R.expression_haqian),
    Expression('鄙视', R.expression_bishi),
    Expression('委屈', R.expression_weiqu),
    Expression('快哭了', R.expression_kuaikule),
    Expression('阴险', R.expression_yinxian),
    Expression('亲亲', R.expression_qinqin),
    Expression('可怜', R.expression_kelian),
    Expression('菜刀', R.expression_caidao),
    Expression('西瓜', R.expression_xigua),
    Expression('啤酒', R.expression_pijiu),
    Expression('咖啡', R.expression_kafei),
    Expression('猪头', R.expression_zhutou),
    Expression('玫瑰', R.expression_meigui),
    Expression('凋谢', R.expression_diaoxie),
    Expression('嘴唇', R.expression_zuichun),
    Expression('爱心', R.expression_aixin),
    Expression('心碎', R.expression_xinsui),
    Expression('蛋糕', R.expression_dangao),
    Expression('炸弹', R.expression_zhadan),
    Expression('便便', R.expression_bianbian),
    Expression('月亮', R.expression_yueliang),
    Expression('太阳', R.expression_taiyang),
    Expression('拥抱', R.expression_yongbao),
    Expression('强', R.expression_qiang),
    Expression('弱', R.expression_ruo),
    Expression('握手', R.expression_woshou),
    Expression('胜利', R.expression_shengli),
    Expression('抱拳', R.expression_baoquan),
    Expression('勾引', R.expression_gouyin),
    Expression('拳头', R.expression_quantou),
    Expression('OK', R.expression_ok),
    Expression('跳跳', R.expression_tiaotiao),
    Expression('发抖', R.expression_fadou),
    Expression('怄火', R.expression_ohuo),
    Expression('转圈', R.expression_zhuanquan),
    Expression('嘿哈', R.expression_heiha),
    Expression('捂脸', R.expression_wulian),
    Expression('奸笑', R.expression_jianxiao),
    Expression('机智', R.expression_jizhi),
    Expression('皱眉', R.expression_zhoumei),
    Expression('耶', R.expression_ye),
    Expression('蜡烛', R.expression_lazhu),
    Expression('红包', R.expression_hongbao),
    Expression('吃瓜', R.expression_chigua),
    Expression('加油', R.expression_jiayou),
    Expression('汗', R.expression_han),
    Expression('天啊', R.expression_tiana),
    Expression('Emm', R.expression_emm),
    Expression('社会社会', R.expression_shehuishehui),
    Expression('旺柴', R.expression_wangchai),
    Expression('好的', R.expression_haode),
    Expression('打脸', R.expression_dalian),
    Expression('加油加油', R.expression_jiayoujiayou),
    Expression('哇', R.expression_wa),
    Expression('發', R.expression_fa),
    Expression('福', R.expression_fu),
    Expression('鸡', R.expression_ji),
  ];

  ///kv
  static final Map<String, String> expressionKV = {};

  ///初始化
  static void init() {
    for (final value in expressionPath) {
      expressionKV[value.name] = value.path;
    }
  }

  ///格式化消息
  // static String format(String msg) {}

  ///表情总长度
  // ignore: non_constant_identifier_names
  static final int EXPRESSION_SIZE = expressionPath.length;
}

///带有表情的文本 ---  表情好为[]中带 key, 如[微笑]
class XExpressionText extends StatelessWidget {
  final String _text;
  final TextStyle style;

  //最大行数,默认-1,不限制
  final int maxLines;
  final TextOverflow? overflow;

  const XExpressionText(
    this._text, {
    Key? key,
    required this.style,
    this.maxLines = -1,
    this.overflow,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      child: (maxLines > 0)
          ? Text.rich(
              TextSpan(
                children: _getContent(),
              ),
              maxLines: maxLines,
              overflow: overflow,
            )
          : Text.rich(TextSpan(
              children: _getContent(),
            )),
    );
  }

  ///使用正则解析表情文本,使用了Text.rich替换掉了Wrap
  _getContent() {
    if (ExpressionData.expressionKV.isEmpty) {
      ExpressionData.init();
    }
    List<InlineSpan> stack = [];
    List<int> indexList = [];

    //正则校验是否含有表情
    RegExp exp = RegExp(r'\[.{1,4}?\]');
    if (exp.hasMatch(_text)) {
      final array = exp.allMatches(_text).toList();
      for (RegExpMatch r in array) {
        final substring = _text.substring(r.start, r.end);
        final select = substring.substring(1, substring.length - 1);
        if (ExpressionData.expressionKV.containsKey(select)) {
          indexList.add(r.start);
          indexList.add(r.end);
        }
      }

      if (indexList.isEmpty) {
        /// 如果说有方括号中的字符都无法解析出来,直接通过文字显示就可以了
        stack.add(
          TextSpan(
            text: _text,
            style: style,
          ),
        );
      } else {
        int afterX = 0;
        for (int x = 0; x < indexList.length; x = x + 2) {
          /// 显示出未解析的纯文字部分
          int y = x + 1;
          final indexX = indexList[x];
          final indexY = indexList[y];
          final substring = _text.substring(afterX, indexX);
          afterX = indexY;
          stack.add(TextSpan(
            text: substring,
            style: style,
          ));

          /// 显示出表情部分
          final xy = _text.substring(indexX, indexY);
          final selectXy = xy.substring(1, xy.length - 1);
          final expressionKV = ExpressionData.expressionKV[selectXy]!;
          stack.add(WidgetSpan(
            child: XImageAsset(
              expressionKV,
              width: 40,
              height: 40,
              package: HConfig.package,
            ),
          ));
        }

        /// 最后部分
        final last = _text.substring(afterX);
        if (U.isNotEmpt(last)) {
          stack.add(TextSpan(
            text: last,
            style: style,
          ));
        }
      }
    } else {
      stack.add(TextSpan(
        text: _text,
        style: style,
      ));
    }
    return stack;
  }
}

/// 显示标签的输入框(使用ExtendedTextField--传入specialTextSpanBuilder)
class XSpecialTextSpanBuilder extends SpecialTextSpanBuilder {
  @override
  SpecialText? createSpecialText(String flag, {TextStyle? textStyle, onTap, int? index}) {
    if (isStart(flag, '[')) {
      return XEmojiText(textStyle!, start: index);
    }
    return null;
  }
}

/// 显示表情的 Span
class XEmojiText extends SpecialText {
  final int? start;

  XEmojiText(TextStyle textStyle, {required this.start}) : super('[', ']', textStyle);

  @override
  InlineSpan finishText() {
    final String key = toString();
    final selectXy = key.substring(1, key.length - 1);
    final expressionKV = ExpressionData.expressionKV[selectXy];

    if (expressionKV != null) {
      return ImageSpan(
        AssetImage(expressionKV, package: HConfig.package),
        imageHeight: D.h(40),
        imageWidth: D.w(40),
        actualText: key,
        start: start!,
        fit: BoxFit.fill,
      );
    } else {
      return TextSpan(text: key, style: textStyle);
    }
  }
}
