import 'package:flutter/material.dart';
import 'package:hgg_view/hgg_view.dart';
import 'package:hgg_view/src/theme/h_colors.dart';

/// tabbar 点击无阴影
class XTabBar extends StatelessWidget implements PreferredSizeWidget {
  const XTabBar({
    Key? key,
    required this.tabs,
  }) : super(key: key);

  final List<String> tabs;

  @override
  Widget build(BuildContext context) {
    return Theme(
      data: Theme.of(context).copyWith(
        splashColor: Colors.transparent,
        highlightColor: Colors.transparent,
      ),
      child: TabBar(
        labelColor: HColors.app_accent_5A46BE,
        unselectedLabelColor: HColors.FF_8F94A5,
        labelStyle:
            TextStyle(fontSize: D.sp(28), fontWeight: XFontWeight.medium),
        unselectedLabelStyle:
            TextStyle(fontSize: D.sp(28), fontWeight: XFontWeight.medium),
        isScrollable: false,
        indicatorSize: TabBarIndicatorSize.tab,
        indicatorPadding: P.only(left: 50, right: 50, bottom: 10),
        indicator: XUnderlineTabIndicator(
            indicatorColor: HColors.app_accent_5A46BE, indicatorWeight: 5),
        // indicatorColor: XColors.app_accent_5A46BE,
        // indicatorWeight: D.h(5),
        labelPadding: P.all(0),
        tabs: tabs.map((tab) => Tab(text: tab)).toList(),
      ),
    );
  }

  @override
  Size get preferredSize => const Size.fromHeight(46);
}

/// tabbar 指示线圆角
class XUnderlineTabIndicator extends Decoration {
  XUnderlineTabIndicator({
    double indicatorWeight = 5,
    Color indicatorColor = Colors.white,
    this.insets = EdgeInsets.zero,
  }) : borderSide = BorderSide(
          width: D.w(indicatorWeight),
          color: indicatorColor,
        );

  /// The color and weight of the horizontal line drawn below the selected tab.
  final BorderSide borderSide;

  /// Locates the selected tab's underline relative to the tab's boundary.
  ///
  /// The [TabBar.indicatorSize] property can be used to define the tab
  /// indicator's bounds in terms of its (centered) tab widget with
  /// [TabBarIndicatorSize.label], or the entire tab with
  /// [TabBarIndicatorSize.tab].
  final EdgeInsetsGeometry insets;

  @override
  Decoration? lerpFrom(Decoration? a, double t) {
    if (a is UnderlineTabIndicator) {
      return UnderlineTabIndicator(
        borderSide: BorderSide.lerp(a.borderSide, borderSide, t),
        insets: EdgeInsetsGeometry.lerp(a.insets, insets, t)!,
      );
    }
    return super.lerpFrom(a, t);
  }

  @override
  Decoration? lerpTo(Decoration? b, double t) {
    if (b is UnderlineTabIndicator) {
      return UnderlineTabIndicator(
        borderSide: BorderSide.lerp(borderSide, b.borderSide, t),
        insets: EdgeInsetsGeometry.lerp(insets, b.insets, t)!,
      );
    }
    return super.lerpTo(b, t);
  }

  @override
  BoxPainter createBoxPainter([VoidCallback? onChanged]) {
    return _UnderlinePainter(this, onChanged);
  }

  Rect _indicatorRectFor(Rect rect, TextDirection textDirection) {
    final Rect indicator = insets.resolve(textDirection).deflateRect(rect);
    return Rect.fromLTWH(
      indicator.left,
      indicator.bottom - borderSide.width,
      indicator.width,
      borderSide.width,
    );
  }

  @override
  Path getClipPath(Rect rect, TextDirection textDirection) {
    return Path()..addRect(_indicatorRectFor(rect, textDirection));
  }
}

class _UnderlinePainter extends BoxPainter {
  _UnderlinePainter(this.decoration, VoidCallback? onChanged)
      : super(onChanged);

  final XUnderlineTabIndicator decoration;

  @override
  void paint(Canvas canvas, Offset offset, ImageConfiguration configuration) {
    assert(configuration.size != null);
    final Rect rect = offset & configuration.size!;
    final TextDirection textDirection = configuration.textDirection!;
    final Rect indicator = decoration
        ._indicatorRectFor(rect, textDirection)
        .deflate(decoration.borderSide.width / 2.0);
    final Paint paint = decoration.borderSide.toPaint()
      ..strokeCap = StrokeCap.round;
    canvas.drawLine(indicator.bottomLeft, indicator.bottomRight, paint);
  }
}
